@@ -1,5 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
+ * Copyright (C) 2005       Imendio AB
  * Copyright (C) 2001-2003  CodeFactory AB
  * Copyright (C) 2001-2003  Richard Hult <richard@imendio.com>
  * Copyright (C) 2001-2002  Mikael Hallendal <micke@imendio.com>
@@ -34,7 +35,7 @@
 #include <libxml/xmlmemory.h>
 #include "mrp-private.h"
 #include "mrp-error.h"
-#include "mrp-intl.h"
+#include <glib/gi18n.h>
 #include "mrp-task.h"
 #include "mrp-resource.h"
 #include "mrp-group.h"
@@ -479,7 +480,7 @@ mpp_write_task_cb (MrpTask *task, MrpParser *parser)
 	xmlNodePtr     node, parent_node;
 	gchar         *name;
 	gchar         *note;
-	mrptime        start, finish;
+	mrptime        start, finish, work_start;
 	MrpConstraint *constraint;
 	gint           duration;
 	gint           work;
@@ -518,6 +519,8 @@ mpp_write_task_cb (MrpTask *task, MrpParser *parser)
 		      "sched", &sched,
 		      NULL);
 
+	work_start = mrp_task_get_work_start (task);
+
 	if (type == MRP_TASK_TYPE_MILESTONE) {
 		finish = start;
 		work = 0;
@@ -535,13 +538,12 @@ mpp_write_task_cb (MrpTask *task, MrpParser *parser)
 	
 	mpp_xml_set_date (node, "start", start);
 	mpp_xml_set_date (node, "end", finish);
+	mpp_xml_set_date (node, "work-start", work_start);
 
 	mpp_xml_set_int (node, "percent-complete", complete);
-
 	mpp_xml_set_int (node, "priority", priority);
 
 	mpp_xml_set_task_type (node, "type", type);
-
 	mpp_xml_set_task_sched (node, "scheduling", sched);
 
 	mpp_write_custom_properties (parser, node, MRP_OBJECT (task));
@@ -1135,7 +1137,7 @@ mrp_parser_to_xml (MrpStorageMrproject  *module,
 {
 	xmlDocPtr  doc;
 	xmlChar   *buf;
-	gsize      len;
+	int        len;
 	
 	g_return_val_if_fail (MRP_IS_STORAGE_MRPROJECT (module), FALSE);
 
