@@ -47,7 +47,7 @@ struct _MrpTaskManagerPriv {
 	gboolean    needs_recalc;
 	gboolean    in_recalc;
 
-	GList      *depencency_list;
+	GList      *dependency_list;
 };
 
 typedef struct {
@@ -76,6 +76,12 @@ static void
 task_manager_task_duration_notify_cb      (MrpTask             *task,
 					   GParamSpec          *spec,
 					   MrpTaskManager      *manager);
+#ifdef WITH_SIMPLE_PRIORITY_SCHEDULING
+static void
+task_manager_task_priority_notify_cb      (MrpTask             *task,
+					   GParamSpec          *spec,
+					   MrpTaskManager      *manager);
+#endif
 static void
 task_manager_task_constraint_notify_cb    (MrpTask             *task,
 					   GParamSpec          *spec,
@@ -115,6 +121,12 @@ task_manager_dump_task_tree               (GNode               *node);
 
 static GObjectClass *parent_class;
 
+static mrptime
+task_manager_calculate_task_start_from_finish (MrpTaskManager *manager,
+				    		MrpTask        *task,
+						mrptime         finish,
+						gint           *duration);
+
 
 GType
 mrp_task_manager_get_type (void)
@@ -261,6 +273,13 @@ task_manager_task_connect_signals (MrpTaskManager *manager,
 			  "notify::duration",
 			  G_CALLBACK (task_manager_task_duration_notify_cb),
 			  manager);
+#ifdef WITH_SIMPLE_PRIORITY_SCHEDULING
+	/* Added to manage the transaction from normal to dominant. */
+	g_signal_connect (task,
+			  "notify::priority",
+			  G_CALLBACK (task_manager_task_priority_notify_cb),
+			  manager);
+#endif
 	g_signal_connect (task,
 			  "notify::constraint",
 			  G_CALLBACK (task_manager_task_constraint_notify_cb),
@@ -883,6 +902,35 @@ remove_predecessor_from_dependency_graph (MrpTaskManager *manager,
 	remove_predecessor_from_dependency_graph_recursive (task, predecessor);
 }
 
+static void
+remove_parent_predecessors_from_dependency_graph (MrpTask *task,
+						  MrpTask *parent)
+{
+	MrpTask            *predecessor;
+	MrpTaskGraphNode   *predecessor_node;
+	MrpTaskGraphNode   *task_node;
+	GList              *list, *l;
+	MrpRelation        *relation;
+	
+	/* Remove parent's predecessors from task and all of its children. */
+	list = imrp_task_peek_predecessors (parent);
+	for (l = list; l; l = l->next) {
+		relation = l->data;
+		predecessor = mrp_relation_get_predecessor (relation);
+
+		/* Remove predecessor from task */
+		predecessor_node = imrp_task_get_graph_node (predecessor);
+		predecessor_node->next = g_list_remove (predecessor_node->next, task);
+		task_node = imrp_task_get_graph_node (task);
+		task_node->prev = g_list_remove (task_node->prev, predecessor);
+			
+		/* Remove predecessor from all its children */
+		if (mrp_task_get_n_children (task) > 0) {
+			remove_predecessor_from_dependency_graph_recursive (task, predecessor);
+		}
+	}
+}
+
 static void
 remove_parent_from_dependency_graph (MrpTaskManager *manager,
 				     MrpTask        *task,
@@ -899,6 +947,40 @@ remove_parent_from_dependency_graph (MrpTaskManager *manager,
 
 	task_node->next = g_list_remove (task_node->next, parent);
 	parent_node->prev = g_list_remove (parent_node->prev, task);
+
+	/* Remove dependencies from the parent's predecessors from the task and all
+	 * its children.
+	 */
+	remove_parent_predecessors_from_dependency_graph (task, parent);
+}
+
+static void
+add_parent_predecessors_to_dependency_graph (MrpTask *task,
+					     MrpTask *parent)
+{
+	MrpTask            *predecessor;
+	MrpTaskGraphNode   *predecessor_node;
+	MrpTaskGraphNode   *task_node;
+	GList              *list, *l;
+	MrpRelation        *relation;
+
+	/* Add parent's predecessors to task and all of its children. */
+	list = imrp_task_peek_predecessors (parent);
+	for (l = list; l; l = l->next) {
+		relation = l->data;
+		predecessor = mrp_relation_get_predecessor (relation);
+
+		/* Add predecessor to task */
+		predecessor_node = imrp_task_get_graph_node (predecessor);
+		predecessor_node->next = g_list_append (predecessor_node->next, task);
+		task_node = imrp_task_get_graph_node (task);
+		task_node->prev = g_list_append (task_node->prev, predecessor);
+
+		/* Add predecessor to all its children */
+		if (mrp_task_get_n_children (task) > 0) {
+			add_predecessor_to_dependency_graph_recursive (task, predecessor);
+		}
+	}
 }
 
 static void
@@ -917,6 +999,11 @@ add_parent_to_dependency_graph (MrpTaskManager *manager,
 
 	task_node->next = g_list_append (task_node->next, parent);
 	parent_node->prev = g_list_append (parent_node->prev, task);
+
+	/* Add dependencies from the parent's predecessors to the task and all
+	 * its children.
+	 */
+	add_parent_predecessors_to_dependency_graph (task, parent);
 }
 
 static void
@@ -1083,8 +1170,8 @@ task_manager_build_dependency_graph (MrpTaskManager *manager)
 		}
 	}
 
-	g_list_free (priv->depencency_list);
-	priv->depencency_list = g_list_reverse (deps);
+	g_list_free (priv->dependency_list);
+	priv->dependency_list = g_list_reverse (deps);
 
 	g_list_free (queue);
 	g_list_free (tasks);
@@ -1098,78 +1185,23 @@ task_manager_build_dependency_graph (MrpTaskManager *manager)
 	manager->priv->needs_recalc = TRUE;
 }
 
-/* Calcluate the earliest start time that a particular predesessor relation
- * allows given.
- */
-static mrptime
-task_manager_calc_relation (MrpTask	*task,
-			    MrpRelation	*relation,
-			    MrpTask	*predecessor)
-{
-	MrpRelationType type;
-	mrptime         time;
-	/*mrptime         start, finish;*/
-	
-	/* FIXME: This does not work correctly for FF and SF. The problem is
-	 * that the start and finish times of task is not known at this stage,
-	 * so we can't really use them.
-	 */
-
-	type = mrp_relation_get_relation_type (relation);
-	
-	switch (type) {
-#if 0
-	case MRP_RELATION_FF:
-		/* finish-to-finish */
-		start =  mrp_task_get_start (task);
-		finish =  mrp_task_get_finish (task);
-
-		time = mrp_task_get_finish (predecessor) +
-			mrp_relation_get_lag (relation) - (finish - start);
-
-		break;
-		
-	case MRP_RELATION_SF:
-		/* start-to-finish */
-		start = mrp_task_get_start (task);
-		finish = mrp_task_get_finish (task);
-
-		time = mrp_task_get_start (predecessor) +
-			mrp_relation_get_lag (relation) - (finish - start);
-		break;
-#endif	
-	case MRP_RELATION_SS:
-		/* start-to-start */
-		time = mrp_task_get_start (predecessor) +
-			mrp_relation_get_lag (relation);
-		break;
-			
-	case MRP_RELATION_FS:
-	case MRP_RELATION_NONE:
-	default:
-		/* finish-to-start */
-		time = mrp_task_get_finish (predecessor) +
-			mrp_relation_get_lag (relation);
-		break;
-	}
-
-	return time;
-}
-
 /* Calculate the start time of the task by finding the latest finish of it's
  * predecessors (plus any lag). Also take constraints into consideration.
  */
 static mrptime
 task_manager_calculate_task_start (MrpTaskManager *manager,
-				   MrpTask        *task)
+				   MrpTask        *task, 
+				   gint           *duration)
 {
 	MrpTaskManagerPriv *priv;
 	MrpTask            *tmp_task;
 	GList              *predecessors, *l;
 	MrpRelation        *relation;
+	MrpRelationType     type;
 	MrpTask            *predecessor;
 	mrptime             project_start;
 	mrptime             start;
+	mrptime             finish;
 	mrptime             dep_start;
 	MrpConstraint       constraint;
 
@@ -1185,10 +1217,51 @@ task_manager_calculate_task_start (MrpTaskManager *manager,
 			relation = l->data;
 			predecessor = mrp_relation_get_predecessor (relation);
 
-			dep_start = task_manager_calc_relation (task,
-								relation,
-								predecessor);
+			type = mrp_relation_get_relation_type (relation);
 			
+			switch (type) {
+			case MRP_RELATION_FF:
+				/* finish-to-finish */
+				/* predecessor must finish before successor can finish */
+				finish = mrp_task_get_finish (predecessor) + mrp_relation_get_lag (relation);
+				start =  task_manager_calculate_task_start_from_finish (manager,
+											task,
+											finish,
+											duration);
+				dep_start = start;
+
+				break;
+				
+			case MRP_RELATION_SF:
+				/* start-to-finish */
+				/* predecessor must start before successor can finish */
+				finish = mrp_task_get_start (predecessor);
+				start =  task_manager_calculate_task_start_from_finish (manager,
+											task,
+											finish,
+											duration);
+
+				dep_start = mrp_task_get_start (predecessor) +
+					    mrp_relation_get_lag (relation) - (finish - start);
+				break;
+
+			case MRP_RELATION_SS:
+				/* start-to-start */
+				/* predecessor must start before successor can start */
+				dep_start = mrp_task_get_start (predecessor) +
+					    mrp_relation_get_lag (relation);
+				break;
+
+			case MRP_RELATION_FS:
+			case MRP_RELATION_NONE:
+			default:
+				/* finish-to-start */
+				/* predecessor must finish before successor can start */
+				dep_start = mrp_task_get_finish (predecessor) +
+					mrp_relation_get_lag (relation);
+				break;
+			}
+
 			start = MAX (start, dep_start);
 		}
 
@@ -1196,7 +1269,7 @@ task_manager_calculate_task_start (MrpTaskManager *manager,
 	}
 
 	/* Take constraint types in consideration. */
-	constraint = impr_task_get_constraint (task);
+	constraint = imrp_task_get_constraint (task);
 	switch (constraint.type) {
 	case MRP_CONSTRAINT_SNET:
 		/* Start-no-earlier-than. */
@@ -1222,20 +1295,13 @@ task_manager_calculate_task_start (MrpTaskManager *manager,
 	return start;
 }
 
-typedef struct {
-	gboolean is_start;
-	mrptime  start;
-	mrptime  end;
-	gint     units;
-} UnitsInterval;
-
-#define UNIT_IVAL_GET_TIME(R) ((R->is_start?R->start:R->end))
-
+/* NOTE: MrpUnitsInterval moved in mrp-task.h to enable
+         other objects to use it. */
 static gint                                         
 units_interval_sort_func (gconstpointer a, gconstpointer b)
 {
-	UnitsInterval *ai = *(UnitsInterval **) a;
-	UnitsInterval *bi = *(UnitsInterval **) b;
+	MrpUnitsInterval *ai = *(MrpUnitsInterval **) a;
+	MrpUnitsInterval *bi = *(MrpUnitsInterval **) b;
 	mrptime       at, bt;
 
 	if (ai->is_start) {
@@ -1260,12 +1326,12 @@ units_interval_sort_func (gconstpointer a, gconstpointer b)
 	}
 }
 
-static UnitsInterval *
+static MrpUnitsInterval *
 units_interval_new (MrpInterval *ival, gint units, gboolean is_start)
 {
-	UnitsInterval *unit_ival;
+	MrpUnitsInterval *unit_ival;
 
-	unit_ival = g_new (UnitsInterval, 1);
+	unit_ival = g_new (MrpUnitsInterval, 1);
 	unit_ival->is_start = is_start;
 	unit_ival->units = units;
 
@@ -1292,50 +1358,184 @@ task_manager_get_task_units_intervals (MrpTaskManager *manager,
 	MrpDay             *day;
 	GList              *ivals, *l;
 	MrpInterval        *ival;
-	UnitsInterval      *unit_ival, *new_unit_ival;
+
+	MrpUnitsInterval   *unit_ival, *new_unit_ival;
+	MrpUnitsInterval   *unit_ival_start, *unit_ival_end;
 	GList              *unit_ivals = NULL;
 	MrpAssignment      *assignment;
 	MrpResource        *resource;
 	GList              *assignments, *a;
-	gint                units;
+	gint                units, units_full, units_orig, priority;
+	mrptime             i_start, i_end;
+
 	mrptime             t;
 	mrptime             poc;
 	GPtrArray          *array;
 	guint               len;
 	gint                i;
 
+	mrptime             diffe;
+
+	gint     res_n;
+
+#ifdef WITH_SIMPLE_PRIORITY_SCHEDULING
+	MrpInterval        *ival_split;
+	MrpUnitsInterval   *unit_ival_start_cmp;
+	MrpUnitsInterval   *split_unit_ival;
+	MrpAssignment      *v_assignment;
+	MrpResource        *v_resource;
+	GList              *v_assignments, *v_a;
+	gint                v_units;
+	GList              *v_tasks, *v_l;
+	GPtrArray          *array_split;
+	gint                e, lastct;
+	mrptime             v_start, v_end;
+	mrptime             i_start_post, i_end_post, i_start_cmp, i_end_cmp;
+#endif
+
 	priv = manager->priv;
 
 	assignments = mrp_task_get_assignments (task);
 
 	array = g_ptr_array_new ();
+	priority    = mrp_task_get_priority (task);
+	
+#ifdef WITH_SIMPLE_PRIORITY_SCHEDULING
+	v_tasks = mrp_task_manager_get_all_tasks (manager);
+#endif
 	
 	for (a = assignments; a; a = a->next) {
 		assignment = a->data;
 		
 		resource = mrp_assignment_get_resource (assignment);
-		units = mrp_assignment_get_units (assignment);
+		units_orig = mrp_assignment_get_units (assignment);
 
 		calendar = mrp_resource_get_calendar (resource);
 		if (!calendar) {
 			calendar = mrp_project_get_calendar (priv->project);
 		}
-
 		day = mrp_calendar_get_day (calendar, date, TRUE);
 		ivals = mrp_calendar_day_get_intervals (calendar, day, TRUE);
 
 		for (l = ivals; l; l = l->next) {
 			ival = l->data;
+			mrp_interval_get_absolute (ival, date, &i_start, &i_end);
+			units = units_orig;
+			diffe = i_end - i_start;
+			
+#ifdef WITH_SIMPLE_PRIORITY_SCHEDULING
+			for (v_l = v_tasks; v_l; v_l = v_l->next) {
+				MrpTask *v_task;
+				
+				v_task =  v_l->data;
+				
+				if (v_task == task) {
+					continue;
+				}
 
+				if (mrp_task_is_dominant (v_task) == FALSE) {
+					continue;
+				}
+
+				/* If intervals not overlapped -> contine. */
+				v_start = mrp_task_get_work_start (v_task);
+				v_end = mrp_task_get_finish (v_task);
+					
+				if (i_start > v_end || i_end < v_start) {
+					continue;
+				}
+
+				/* Compare resources task with resources of dominant task. */
+				v_assignments = mrp_task_get_assignments (v_task);
+
+				for (v_a = v_assignments; v_a; v_a = v_a->next) {
+					v_assignment = v_a->data;
+		
+					v_resource = mrp_assignment_get_resource (v_assignment);
+					if (v_resource == resource) {
+						v_units = mrp_assignment_get_units (v_assignment);
+						/* 
+						   If the dominant cost is compatible with the task
+						   request -> break. 
+
+						   FIXME - tasks that share the vampirised resource not work!
+						*/
+						if (100 - v_units > units) {
+							break;
+						}
+
+						/* Trim the interval of the dominant task. */
+						v_start = (v_start < i_start ? 
+								   i_start : v_start);
+						v_end = (v_end > i_end ?
+								 i_end : v_end);
+						
+						if (i_start < v_start) {
+							/*
+							     ----...
+							   ------...
+							   ival len from start to dominant
+							*/
+							ival = mrp_interval_new (i_start-date, v_start-date);
+
+							unit_ival_start = units_interval_new (ival, units, TRUE);
+							unit_ival_start->units_full = units;
+							unit_ival_end = units_interval_new (ival, units, FALSE);
+							unit_ival_end->units_full = units;
+							g_ptr_array_add (array, unit_ival_start);
+							g_ptr_array_add (array, unit_ival_end);
+						}
+						
+						ival = mrp_interval_new (v_start-date, v_end-date);
+
+						unit_ival_start = units_interval_new (ival, (100 - v_units), TRUE);
+						unit_ival_start->units_full = units;
+						unit_ival_end = units_interval_new (ival, (100 - v_units), FALSE);
+						unit_ival_end->units_full = units;						
+						g_ptr_array_add (array, unit_ival_start);
+						g_ptr_array_add (array, unit_ival_end);
+
+						if (v_end < i_end) {
+							/*
+							   ----  ...
+							   ------...
+							   ival len from end to dominant
+							*/
+							ival = mrp_interval_new (v_end-date, i_end-date);
+
+							unit_ival_start = units_interval_new (ival, units, TRUE);
+							unit_ival_start->units_full = units;
+							unit_ival_end = units_interval_new (ival, units, FALSE);
+							unit_ival_end->units_full = units;
+							g_ptr_array_add (array, unit_ival_start);
+							g_ptr_array_add (array, unit_ival_end);
+						}
+						break;
+					}
+				} /* for (v_a = v_assignments; v_a; ... */
+				if (v_a != NULL) {
+					break;
+				}
+			} /* for (v_l = v_tasks; v_l; ... */
+			
+	
+			if (v_l == NULL) {
+#endif /* ifdef WITH_SIMPLE_PRIORITY_SCHEDULING */
 			/* Start of the interval. */
-			unit_ival = units_interval_new (ival, units, TRUE);
-			g_ptr_array_add (array, unit_ival);
+				unit_ival_start = units_interval_new (ival, units, TRUE);
+				unit_ival_start->units_full = units;
 
 			/* End of the interval. */
-			unit_ival = units_interval_new (ival, units, FALSE);
-			g_ptr_array_add (array, unit_ival);
-		}
-	}
+				unit_ival_end = units_interval_new (ival, units, FALSE);
+				unit_ival_end->units_full = units;
+
+				g_ptr_array_add (array, unit_ival_start);
+				g_ptr_array_add (array, unit_ival_end);
+#ifdef WITH_SIMPLE_PRIORITY_SCHEDULING
+			}
+#endif
+		} /* for (l = ivals; l; ... */
+	} /* for (a = assignments; a; ... */
 
 	/* If the task is not allocated, we handle it as if we have one resource
 	 * assigned to it, 100%, using the project calendar.
@@ -1351,20 +1551,82 @@ task_manager_get_task_units_intervals (MrpTaskManager *manager,
 			
 			/* Start of the interval. */
 			unit_ival = units_interval_new (ival, 100, TRUE);
+			unit_ival->units_full = 100;
 			g_ptr_array_add (array, unit_ival);
 
 			/* End of the interval. */
 			unit_ival = units_interval_new (ival, 100, FALSE);
+			unit_ival->units_full = 100;
 			g_ptr_array_add (array, unit_ival);
 		}
 	}
 	
+#ifdef WITH_SIMPLE_PRIORITY_SCHEDULING
+	/* Requantize the time intervals. */
+	array_split = g_ptr_array_new ();
+	len = array->len;
+	poc = -1;
+	units = 0;
+	/* += 2 because start and end are strictly joined. */
+	for (i = 0; i < len; i+= 2) {
+		unit_ival_start = g_ptr_array_index (array, i);
+
+		i_start = unit_ival_start->start;
+		i_end = unit_ival_start->end;
+
+		lastct = 1;
+		i_end_post = i_end;
+		for (i_start_post = i_start; i_start_post < i_end && lastct > 0;) {
+			lastct = 0;
+			i_end_post = i_end;
+			for (e = 0 ; e < len ; e+= 2) {
+				unit_ival_start_cmp = g_ptr_array_index (array, e);
+				
+				i_start_cmp = unit_ival_start_cmp->start;
+				i_end_cmp = unit_ival_start_cmp->end;
+
+				/* If not overlapped intervals. */
+				if (i_start_post >= i_end_cmp || i_end_post <= i_start_cmp) {
+					continue;
+				}
+				if (i_start_post < i_start_cmp && i_start_cmp < i_end_post) {
+					i_end_post = i_start_cmp;
+					lastct++;
+					continue;
+				}
+				if (i_start_post < i_end_cmp && i_end_cmp < i_end_post) {
+					i_end_post = i_end_cmp;
+					lastct++;
+					continue;
+				}
+			}
+			ival_split = mrp_interval_new (i_start_post, i_end_post);
+			split_unit_ival = units_interval_new (ival_split, unit_ival_start->units, TRUE);
+			split_unit_ival->units_full = unit_ival_start->units_full;
+			g_ptr_array_add (array_split, split_unit_ival);
+			split_unit_ival = units_interval_new (ival_split, unit_ival_start->units, FALSE);
+			split_unit_ival->units_full = unit_ival_start->units_full;
+			g_ptr_array_add (array_split, split_unit_ival);
+			i_start_post = i_end_post;
+		}
+	}
+	for (i = 0; i < array->len; i++) {
+		g_free (array->pdata[i]);
+	}
+	
+	g_ptr_array_free (array, TRUE);
+	array = array_split;
+#endif /* ifdef WITH_SIMPLE_PRIORITY_SCHEDULING */
+	/* Clean and reassign the split_array ptr to the array */
 	g_ptr_array_sort (array, units_interval_sort_func);
 
 	len = array->len;
 
+
 	poc = -1;
 	units = 0;
+	units_full = 0;
+	res_n = 0;
 	for (i = 0; i < len; i++) {
 		unit_ival = g_ptr_array_index (array, i);		
 
@@ -1376,23 +1638,28 @@ task_manager_get_task_units_intervals (MrpTaskManager *manager,
 			 * determined by now.
 			 */
 			if (poc != -1) {
-				if (units > 0) {
-					new_unit_ival = g_new (UnitsInterval, 1);
-					new_unit_ival->units = units;
-					new_unit_ival->start = poc;
-					new_unit_ival->end = t;
-					
-					unit_ivals = g_list_prepend (unit_ivals, new_unit_ival);
-				}
+				new_unit_ival = g_new (MrpUnitsInterval, 1);
+				new_unit_ival->units = units;
+				new_unit_ival->units_full = units_full;
+				new_unit_ival->start = poc;
+				new_unit_ival->end = t;
+				new_unit_ival->res_n = res_n;
+				res_n = 0;
+				unit_ivals = g_list_prepend (unit_ivals, new_unit_ival);
 			}
-				
+			
 			poc = t;
 		}
-
+		
 		if (unit_ival->is_start) {
 			units += unit_ival->units;
+			units_full += unit_ival->units_full;
+			if (assignments) {
+				res_n++;
+			}
 		} else {
 			units -= unit_ival->units;
+			units_full -= unit_ival->units_full;
 		}
 	}
 
@@ -1416,7 +1683,7 @@ task_manager_calculate_milestone_work_start (MrpTaskManager *manager,
 	mrptime             t1, t2;
 	mrptime             work_start;
 	GList              *unit_ivals, *l;
-	UnitsInterval      *unit_ival;
+	MrpUnitsInterval   *unit_ival;
 	MrpTaskType         type;
 	
 	priv = manager->priv;
@@ -1499,11 +1766,11 @@ task_manager_calculate_task_finish (MrpTaskManager *manager,
 	gint                work;
 	gint                effort;
 	gint                delta;
-	GList              *unit_ivals, *l;
-	UnitsInterval      *unit_ival;
+	GList              *unit_ivals, *unit_ivals_tot = NULL, *l = NULL;
+	MrpUnitsInterval   *unit_ival;
 	MrpTaskType         type;
 	MrpTaskSched        sched;
-	
+
 	priv = manager->priv;
 
 	if (task == priv->root) {
@@ -1547,6 +1814,7 @@ task_manager_calculate_task_finish (MrpTaskManager *manager,
 		}
 
 		if (!unit_ivals) {
+			/* Holidays for all. */
 			t += 60*60*24;
 			continue;
 		}
@@ -1556,7 +1824,6 @@ task_manager_calculate_task_finish (MrpTaskManager *manager,
 
 			t1 = t + unit_ival->start;
 			t2 = t + unit_ival->end;
-			
 			/* Skip any intervals before the task starts. */
 			if (t2 < start) {
 				continue;
@@ -1577,20 +1844,35 @@ task_manager_calculate_task_finish (MrpTaskManager *manager,
 			if (sched == MRP_TASK_SCHED_FIXED_WORK) {
 				delta = floor (0.5 + (double) unit_ival->units * (t2 - t1) / 100.0);
 
-				*duration += (t2 - t1);
-				
-				if (effort + delta >= work) {
-					finish = t1 + floor (0.5 + (work - effort) / unit_ival->units * 100.0);
+				if (unit_ival->units_full > 0) {
+					*duration += (t2 - t1);
+				}
 
-					/* Subtract the spill. */
-					*duration -= floor (0.5 + (effort + delta - work) / unit_ival->units * 100.0);
+				if (effort + delta >= work) {
+					/* Subtract the spill to duration. */
+					if (unit_ival->units) {
+						finish = t1 + floor (0.5 + (work - effort) / unit_ival->units * 100.0);
+						*duration -= floor (0.5 + (effort + delta - work) / unit_ival->units * 100.0);
+					} else {
+						finish = t1 + floor (work - effort);
+						*duration -= floor (0.5 + (effort + delta - work));
+					}
+					
+					unit_ival->start = t1;
+					unit_ival->end = finish;
+					unit_ivals_tot = g_list_prepend (unit_ivals_tot, unit_ival);
 					goto done;
 				}
+				unit_ival->start = t1;
+				unit_ival->end = t2;
+				unit_ivals_tot = g_list_prepend (unit_ivals_tot, unit_ival);
 			}
 			else if (sched == MRP_TASK_SCHED_FIXED_DURATION) {
 				delta = t2 - t1;
-				
-				if (effort + delta >= *duration) {
+
+				if (unit_ival->units_full == 0) {
+					delta = 0;
+				} else if (effort + delta >= *duration) {
 					/* Done, make sure we don't spill. */
 					finish = t1 + *duration - effort;
 					goto done;
@@ -1602,7 +1884,6 @@ task_manager_calculate_task_finish (MrpTaskManager *manager,
 			
 			effort += delta;
 		}
-		
 		t += 60*60*24;
 	}
 
@@ -1613,10 +1894,168 @@ task_manager_calculate_task_finish (MrpTaskManager *manager,
 	}
 	imrp_task_set_work_start (task, work_start);
 
+	/* clean the tail of the list before exit of function. */
+	if (l) {
+		for (l = l->next ; l; l = l->next) { 
+			unit_ival = l->data;
+		g_free (unit_ival);
+		}
+	}
+	g_list_free (unit_ivals);
+
+	unit_ivals_tot = g_list_reverse (unit_ivals_tot);
+	mrp_task_set_unit_ivals (task, unit_ivals_tot);
+
+	return finish;
+}
+
+/* Calculate the start time from the work needed for the task, and the effort
+ * that the allocated resources add to the task. Uses the project calendar if no
+ * resources are allocated. This function also sets the work_start property of
+ * the task, which is the first time that actually has work scheduled, this can
+ * differ from the start if start is inside a non-work period.
+ */
+static mrptime
+task_manager_calculate_task_start_from_finish (MrpTaskManager *manager,
+				    MrpTask        *task,
+				    mrptime         finish,
+				    gint           *duration)
+{
+	MrpTaskManagerPriv *priv;
+	mrptime             start;
+	mrptime             t;
+	mrptime             t1, t2;
+	mrptime             work_start;
+	mrptime             project_start;
+	gint                work;
+	gint                effort;
+	gint                delta;
+	GList              *unit_ivals, *l;
+	MrpUnitsInterval   *unit_ival;
+	MrpTaskType         type;
+	MrpTaskSched        sched;
+
+	priv = manager->priv;
+
+	if (task == priv->root) {
+		g_warning ("Tried to get duration of root task.");
+		return 0;
+	}
+
+	effort = 0;
+	start = finish;
+	work_start = -1;
+	t = mrp_time_align_day (start);
+	project_start = mrp_project_get_project_start (priv->project);
+
+
+	/* Milestone tasks can be special cased, no duration. */
+	type = mrp_task_get_task_type (task);
+	if (type == MRP_TASK_TYPE_MILESTONE) {
+		*duration = 0;
+		task_manager_calculate_milestone_work_start (manager, task, start);
+		return start;
+	}
+	
+	work = mrp_task_get_work (task);
+	sched = mrp_task_get_sched (task);
+
+	if (sched == MRP_TASK_SCHED_FIXED_WORK) {
+		*duration = 0;
+	} else {
+		*duration = mrp_task_get_duration (task);
+	}
+
+	while (1) {
+		unit_ivals = g_list_reverse (task_manager_get_task_units_intervals (manager, task, t));
+
+		/* If we don't get anywhere in 100 days, then the calendar must
+		 * be broken, so we abort the scheduling of this task. It's not
+		 * the best solution but fixes the issue for now.
+		 */
+		if (effort == 0 && finish - t > (60*60*24*100)) {
+			break;
+		}
+
+		if (!unit_ivals) {
+			t -= 60*60*24;
+			continue;
+		}
+		
+		for (l = unit_ivals; l; l = l->next) { 
+			unit_ival = l->data;
+
+			t1 = t + unit_ival->start;
+			t2 = t + unit_ival->end;
+			
+			/* Skip any intervals after the task ends. */
+			if (t1 > finish) {
+				continue;
+			}
+
+			/* Don't add time after the finish time of the task. */
+			t2 = MIN (t2, finish);
+
+			if (t1 == t2) {
+				continue;
+			}
+			
+			if (work_start == -1) {
+				work_start = t1;
+			}
+
+			/* Effort added by this interval. */
+			if (sched == MRP_TASK_SCHED_FIXED_WORK) {
+				delta = floor (0.5 + (double) unit_ival->units * (t2 - t1) / 100.0);
+
+				*duration += (t2 - t1);
+				
+				if (effort + delta >= work) {
+					/* Subtract the spill to duration. */
+					if (unit_ival->units) {
+						start = t2 - floor (0.5 + (work - effort) / unit_ival->units * 100.0);
+						*duration -= floor (0.5 + (effort + delta - work) / unit_ival->units * 100.0);
+					} else {
+						start = t2 - floor (0.5 + (work - effort));
+						*duration -= floor (0.5 + (effort + delta - work));
+					}						
+					goto done;
+				}
+			}
+			else if (sched == MRP_TASK_SCHED_FIXED_DURATION) {
+				delta = t2 - t1;
+				
+				if (unit_ival->units_full == 0) {
+					delta = 0;
+				} else if (effort + delta >= *duration) {
+					/* Done, make sure we don't spill. */
+					start = t2 - (*duration - effort);
+					goto done;
+				}
+			} else {
+				/* Schedule is either fixed work of fixed duration - we should never get here */
+				delta = 0;
+				g_assert_not_reached ();
+			}
+			
+			effort += delta;
+		}
+		
+		t -= 60*60*24;
+	}
+
+ done:
+
+	start = MAX (start, project_start);
+	if (work_start == -1) {
+		work_start = start;
+	}
+	imrp_task_set_work_start (task, work_start);
+
 	g_list_foreach (unit_ivals, (GFunc) g_free, NULL);
 	g_list_free (unit_ivals);
 
-	return finish;
+	return start;
 }
 
 static void
@@ -1638,6 +2077,7 @@ task_manager_do_forward_pass_helper (MrpTaskManager *manager,
 	old_start = mrp_task_get_start (task);
 	old_finish = mrp_task_get_finish (task);
 	old_duration = old_finish - old_start;
+	duration = 0;
 	
 	if (mrp_task_get_n_children (task) > 0) {
 		MrpTask *child;
@@ -1645,6 +2085,7 @@ task_manager_do_forward_pass_helper (MrpTaskManager *manager,
 		sub_start = -1;
 		sub_work_start = -1;
 		sub_finish = -1;
+		work = 0;
 				
 		child = mrp_task_get_first_child (task);
 		while (child) {
@@ -1669,6 +2110,7 @@ task_manager_do_forward_pass_helper (MrpTaskManager *manager,
 				sub_work_start = MIN (sub_work_start, t2);
 			}
 
+			work += mrp_task_get_work (child);
 			child = mrp_task_get_next_sibling (child);
 		}
 
@@ -1676,21 +2118,15 @@ task_manager_do_forward_pass_helper (MrpTaskManager *manager,
 		imrp_task_set_work_start (task, sub_work_start);
 		imrp_task_set_finish (task, sub_finish);
 
-		/* FIXME: should work be the sum of the work of the children
-		 * here? The problem is that summary tasks really display
-		 * duration in the work field. That's why we set duration to the
-		 * same value as work below.
-		 */
-		
-		work = mrp_task_manager_calculate_task_work (manager,
+		duration = mrp_task_manager_calculate_summary_duration (manager,
 							     task,
 							     sub_start,
 							     sub_finish);
 		imrp_task_set_work (task, work);
-		imrp_task_set_duration (task, work);
+		imrp_task_set_duration (task, duration);
 	} else {
 		/* Non-summary task. */
-		t1 = task_manager_calculate_task_start (manager, task);
+		t1 = task_manager_calculate_task_start (manager, task, &duration);
 		t2 = task_manager_calculate_task_finish (manager, task, t1, &duration);
 		
 		imrp_task_set_start (task, t1);
@@ -1761,9 +2197,9 @@ task_manager_do_forward_pass (MrpTaskManager *manager,
 	*/
 
 	if (start_task) {
-		l = g_list_find (priv->depencency_list, start_task);
+		l = g_list_find (priv->dependency_list, start_task);
 	} else {
-		l = priv->depencency_list;
+		l = priv->dependency_list;
 	}
 	
 	while (l) {
@@ -1793,7 +2229,7 @@ task_manager_do_backward_pass (MrpTaskManager *manager)
 
 	project_finish = mrp_task_get_finish (priv->root);
 
-	tasks = g_list_reverse (g_list_copy (priv->depencency_list));
+	tasks = g_list_reverse (g_list_copy (priv->dependency_list));
 
 	for (l = tasks; l; l = l->next) {
 		MrpTask *task, *parent;
@@ -1979,6 +2415,15 @@ task_manager_task_duration_notify_cb (MrpTask        *task,
 	mrp_task_manager_recalc (manager, TRUE);
 }
 
+#ifdef WITH_SIMPLE_PRIORITY_SCHEDULING
+static void
+task_manager_task_priority_notify_cb (MrpTask        *task,
+									  GParamSpec     *spec,
+									  MrpTaskManager *manager)
+{
+	mrp_task_manager_recalc (manager, TRUE);
+}
+#endif
 static void
 task_manager_task_constraint_notify_cb (MrpTask        *task,
 					GParamSpec     *spec,
@@ -2103,13 +2548,43 @@ check_predecessor_traverse (MrpTaskManager *manager,
 	
 	node = imrp_task_get_graph_node (task);
 	for (l = node->next; l; l = l->next) {
-		if (!check_predecessor_traverse (manager, l->data, end, length + 1))
+		if (!check_predecessor_traverse (manager, l->data, end, length + 1)) {
 			return FALSE;
+		}
 	}
 	
 	return TRUE;
 }
 
+static gboolean
+check_move_traverse_recursive (MrpTaskManager *manager,
+			       MrpTask *task)
+{
+	MrpTask          *child;
+	gboolean          retval = TRUE;
+	
+	child = mrp_task_get_first_child (task);
+	while (retval && child) {
+		retval = check_predecessor_traverse (manager, child, child, 1);
+
+		if (retval && mrp_task_get_n_children (child) > 0) {
+			retval = check_move_traverse_recursive (manager, child);
+		}
+		
+		child = mrp_task_get_next_sibling (child);
+	}
+
+	return retval;
+}
+
+static gboolean
+check_move_traverse (MrpTaskManager *manager,
+		     MrpTask        *task)
+{
+	return check_predecessor_traverse (manager, task, task, 1) &&
+		check_move_traverse_recursive (manager, task);
+}
+
 gboolean
 mrp_task_manager_check_predecessor (MrpTaskManager  *manager,
 				    MrpTask         *task,
@@ -2168,8 +2643,6 @@ mrp_task_manager_check_move (MrpTaskManager  *manager,
 	g_return_val_if_fail (MRP_IS_TASK (task), FALSE);
 	g_return_val_if_fail (MRP_IS_TASK (parent), FALSE);
 
-	/* FIXME: Check this. */
-
 	/* Remove the task from the old parent and add it to its new parent. */
 	remove_task_from_dependency_graph (manager, task, mrp_task_get_parent (task));
 	add_task_to_dependency_graph (manager, task, parent);
@@ -2185,7 +2658,7 @@ mrp_task_manager_check_move (MrpTaskManager  *manager,
 				   task_manager_unset_visited_func,
 				   NULL);
 	
-	retval = check_predecessor_traverse (manager, task, task, 1);
+	retval = check_move_traverse (manager, task);
 
 	/* Put the task back again. */
 	remove_task_from_dependency_graph (manager, task, parent);
@@ -2257,6 +2730,80 @@ task_manager_get_work_for_calendar (MrpTaskManager *manager,
 	return work;
 }
 
+static gint
+task_manager_get_work_for_task_with_assignments (MrpTaskManager *manager,
+									   MrpTask        *task,
+									   mrptime         start,
+									   mrptime         finish)
+{
+	MrpTaskManagerPriv *priv;
+	mrptime             t;
+	mrptime             t1, t2;
+	gint                work, delta;
+	GList              *ivals, *l;
+	MrpUnitsInterval   *ival;
+
+	priv = manager->priv;
+
+	work = 0;
+
+	/* Loop through the intervals of the calendar and add up the work, until
+	 * the finish time is hit.
+	 */
+	t = mrp_time_align_day (start);
+
+
+	while (t < finish) {
+		ivals = task_manager_get_task_units_intervals (manager, task, t);
+
+		/* If we don't get anywhere in 100 days, then the calendar must
+		 * be broken, so we abort the scheduling of this task. It's not
+		 * the best solution but fixes the issue for now.
+		 */
+		if (work == 0 && t - start > (60*60*24*100)) {
+			break;
+		}
+
+		if (!ivals) {
+			/* Holidays for all. */
+			t += 60*60*24;
+			continue;
+		}
+		
+		for (l = ivals; l; l = l->next) { 
+			ival = l->data;
+
+			t1 = t + ival->start;
+			t2 = t + ival->end;
+			/* Skip any intervals before the task starts. */
+			if (t2 < start) {
+				continue;
+			}
+
+			/* Don't add time before the start of the task. */
+			t1 = MAX (t1, start);
+			if (t1 == t2) {
+				continue;
+			}
+
+			/* Resize the too long interval */
+			t2 = MIN (t2, finish);
+			if (t1 >= t2) {
+				break;
+			}
+
+			delta = floor (0.5 + (double) ival->units * (t2 - t1) / 100.0);
+			
+			work += delta;
+		}
+	
+
+		t += 24*60*60;
+	}
+
+	return (work);
+}
+
 /* Calculate the work needed to achieve the specified start and finish, with the
  * allocated resources' calendars in consideration.
  */
@@ -2268,9 +2815,7 @@ mrp_task_manager_calculate_task_work (MrpTaskManager *manager,
 {
 	MrpTaskManagerPriv *priv;
 	gint                work = 0;
-	MrpAssignment      *assignment;
-	MrpResource        *resource;
-	GList              *assignments, *a;
+	GList              *assignments;
 	MrpCalendar        *calendar;
 
 	priv = manager->priv;
@@ -2296,24 +2841,13 @@ mrp_task_manager_calculate_task_work (MrpTaskManager *manager,
 	 */
 
 	assignments = mrp_task_get_assignments (task);
-	for (a = assignments; a; a = a->next) {
-		assignment = a->data;
-
-		resource = mrp_assignment_get_resource (assignment);
-
-		calendar = mrp_resource_get_calendar (resource);
-		if (!calendar) {
-			calendar = mrp_project_get_calendar (priv->project);
-		}
-
-		work += task_manager_get_work_for_calendar (manager,
-							    calendar,
-							    start,
-							    finish) *
-			mrp_assignment_get_units (assignment) / 100;
+	if (assignments) {
+		work = task_manager_get_work_for_task_with_assignments (manager,
+													  task,
+													  start,
+													  finish);
 	}
-	
-	if (!assignments) {
+	else {
 		calendar = mrp_project_get_calendar (priv->project);
 
 		work = task_manager_get_work_for_calendar (manager,
@@ -2325,3 +2859,76 @@ mrp_task_manager_calculate_task_work (MrpTaskManager *manager,
 	return work;
 }
 
+/* Calculate summary duration using the project calendar,
+   ignoring assignments' calendars
+ */
+gint
+mrp_task_manager_calculate_summary_duration (MrpTaskManager *manager,
+				      MrpTask        *task,
+				      mrptime         start,
+				      mrptime         finish)
+{
+	MrpTaskManagerPriv *priv;
+	mrptime             t;
+	mrptime             t1, t2;
+	gint                duration = 0;
+	MrpCalendar        *calendar;
+	MrpDay             *day;
+	GList              *ivals, *l;
+	MrpInterval        *ival;
+
+	priv = manager->priv;
+
+	if (task == priv->root) {
+		return 0;
+	}
+
+	if (start == -1) {
+		start = mrp_task_get_start (task);
+	}
+
+	if (finish <= start) {
+		return 0;
+	}
+	
+	calendar = mrp_project_get_calendar (priv->project);
+
+	duration = 0;
+
+	/* Loop through the intervals of the calendar and add up the working time, until
+	 * the finish time is hit.
+	 */
+	t = mrp_time_align_day (start);
+
+	while (t < finish) {
+		day = mrp_calendar_get_day (calendar, t, TRUE);
+		ivals = mrp_calendar_day_get_intervals (calendar, day, TRUE);
+
+		for (l = ivals; l; l = l->next) {
+			ival = l->data;
+			
+			mrp_interval_get_absolute (ival, t, &t1, &t2);
+			
+			/* Skip intervals that are before the task. */
+			if (t2 < start) {
+				continue;
+			}
+			
+			/* Stop if the interval starts after the task. */
+			if (t1 >= finish) {
+				break;
+			}
+			
+			/* Don't add time outside the task. */
+			t1 = MAX (t1, start);
+			t2 = MIN (t2, finish);
+			
+			duration += t2 - t1;
+		}
+		
+		t += 24*60*60;
+	}
+
+	return duration;
+}
+
