@@ -341,6 +341,12 @@ task_view_get_widget (PlannerView *view)
 						    COL_SLACK, _("Slack"),
 						    COL_COST, _("Cost"),
 						    COL_ASSIGNED_TO, _("Assigned to"),
+						    /* i18n: The string "% Complete" will be used in the header
+						     * of a column containing values from 0 upto 100, indicating
+						     * what part of a task has been completed. 
+						     * xgettext:no-c-format
+						     */
+						    COL_COMPLETE, _("% Complete"),
 						    -1);
 
 		g_object_unref (model);
@@ -814,6 +820,7 @@ task_view_load_columns (PlannerView *view)
 			    strcmp (id, "duration") == 0 ||
 			    strcmp (id, "slack") == 0 ||
 			    strcmp (id, "cost") == 0 ||
+			    strcmp (id, "complete") == 0 ||
 			    strcmp (id, "assigned-to")) {
 				gtk_tree_view_column_set_visible (column, TRUE);
 			} else {
