@@ -25,20 +25,15 @@
 #include <string.h>
 #include <math.h>
 #include <locale.h>
-#include <sys/types.h>
-#include <sys/stat.h>
+#include <glib/gi18n.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
-#include <libgnome/gnome-help.h>
-#include <glib/gi18n.h>
-#include <libgnomeui/gnome-about.h>
-#include <libgnomeui/gnome-href.h>
-#include <libgnomeui/gnome-stock-icons.h>
 #include <libgnomeprintui/gnome-print-dialog.h>
 #include <libgnomeprintui/gnome-print-job-preview.h>
 #include <libplanner/mrp-error.h>
 #include <libplanner/mrp-project.h>
+#include <libplanner/mrp-paths.h>
 #include <libegg/recent-files/egg-recent-view.h>
 #include <libegg/recent-files/egg-recent-view-uimanager.h>
 #include <libegg/recent-files/egg-recent-util.h>
@@ -46,7 +41,6 @@
 #include "planner-conf.h"
 #include "planner-sidebar.h"
 #include "planner-window.h"
-#include "planner-view-loader.h"
 #include "planner-plugin-loader.h"
 #include "planner-project-properties.h"
 #include "planner-phase-dialog.h"
@@ -54,7 +48,11 @@
 #include "planner-day-type-dialog.h"
 #include "planner-print-dialog.h"
 #include "planner-view.h"
-#include "planner-cmd-manager.h"
+#include "planner-gantt-view.h"
+#include "planner-task-view.h"
+#include "planner-resource-view.h"
+#include "planner-usage-view.h"
+#include "planner-util.h"
 
 #define d(x)
 
@@ -107,6 +105,12 @@ enum {
 	LAST_SIGNAL
 };
 
+/* Links type */
+enum {
+	LINK_TYPE_EMAIL,
+	LINK_TYPE_URL
+};
+
 static void       window_class_init                      (PlannerWindowClass           *klass);
 static void       window_init                            (PlannerWindow                *window);
 static void       window_finalize                        (GObject                      *object);
@@ -196,15 +200,18 @@ static void window_connect_proxy_cb (GtkUIManager  *manager,
 				     GtkWidget     *proxy,
 				     PlannerWindow *window);
 
+static void handle_links (GtkAboutDialog *about G_GNUC_UNUSED, 
+			  const gchar *link,
+			  gpointer data);
 
-#define CONF_MAIN_WINDOW_DIR       "/ui"
-#define CONF_MAIN_WINDOW_MAXIMIZED "/ui/main_window_maximized"
-#define CONF_MAIN_WINDOW_WIDTH     "/ui/main_window_width"
-#define CONF_MAIN_WINDOW_HEIGHT    "/ui/main_window_height"
-#define CONF_MAIN_WINDOW_POS_X     "/ui/main_window_position_x"
-#define CONF_MAIN_WINDOW_POS_Y     "/ui/main_window_position_y"
+#define CONF_WINDOW_DIR       "/ui"
+#define CONF_WINDOW_MAXIMIZED "/ui/main_window_maximized"
+#define CONF_WINDOW_WIDTH     "/ui/main_window_width"
+#define CONF_WINDOW_HEIGHT    "/ui/main_window_height"
+#define CONF_WINDOW_POS_X     "/ui/main_window_position_x"
+#define CONF_WINDOW_POS_Y     "/ui/main_window_position_y"
 #define CONF_ACTIVE_VIEW           "/ui/active_view"
-#define CONF_MAIN_LAST_DIR         "/general/last_dir"
+#define CONF_LAST_DIR         "/general/last_dir"
 
 #define VIEW_PATH "/menu/View/Views placeholder"
 #define VIEW_GROUP "view group"
@@ -296,7 +303,7 @@ static GtkActionEntry entries[] = {
 	  GTK_STOCK_HELP,          N_("_User Guide"),              "F1",                N_("Show the Planner User Guide"),
 	  G_CALLBACK (window_help_cb) },
 	{ "HelpAbout",
-	  GNOME_STOCK_ABOUT,       N_("_About"),                   NULL,                N_("About this application"),
+	  GTK_STOCK_ABOUT,         N_("_About"),                   NULL,                N_("About this application"),
 	  G_CALLBACK (window_about_cb) },
 };
 
@@ -358,10 +365,6 @@ window_init (PlannerWindow *window)
 	priv = g_new0 (PlannerWindowPriv, 1);
 	window->priv = priv;
 
-	gtk_window_set_icon_from_file (GTK_WINDOW (window),
-				       DATADIR "/pixmaps/gnome-planner.png",
-				       NULL);
-
 	/* Setup drag-n-drop. */
 	gtk_drag_dest_set (GTK_WIDGET (window),
 			   GTK_DEST_DEFAULT_ALL,
@@ -443,6 +446,86 @@ window_add_widget (GtkUIManager  *merge,
 	gtk_box_pack_start (GTK_BOX (priv->ui_box), widget, FALSE, FALSE, 0);
 }
 
+static void
+window_add_stock_icon (GtkIconFactory *icon_factory,
+		       const gchar    *stock_id,
+		       const gchar    *filename)
+{
+	gchar      *path;
+	GdkPixbuf  *pixbuf;
+	GtkIconSet *icon_set;
+	
+	path = mrp_paths_get_image_dir (filename);
+	pixbuf = gdk_pixbuf_new_from_file (path, NULL);
+	icon_set = gtk_icon_set_new_from_pixbuf (pixbuf);
+	g_object_unref (pixbuf);
+	gtk_icon_factory_add (icon_factory, stock_id, icon_set);
+	g_free (path);
+}
+
+static void
+window_add_stock_icons (void)
+{
+	static GtkIconFactory *icon_factory = NULL;
+
+	if (icon_factory) {
+		return;
+	}
+	
+	icon_factory = gtk_icon_factory_new ();
+	gtk_icon_factory_add_default (icon_factory);
+
+	/* Task/Gantt icons. */
+	window_add_stock_icon (icon_factory, 
+			       "planner-stock-insert-task",
+			       "24_insert_task.png");
+	
+	window_add_stock_icon (icon_factory, 
+			       "planner-stock-remove-task",
+			       "24_remove_task.png");
+	
+	window_add_stock_icon (icon_factory, 
+			       "planner-stock-unlink-task",
+			       "24_unlink_task.png");
+	
+	window_add_stock_icon (icon_factory, 
+			       "planner-stock-link-task",
+			       "24_link_task.png");
+	
+	window_add_stock_icon (icon_factory, 
+			       "planner-stock-indent-task",
+			       "24_indent_task.png");
+	
+	window_add_stock_icon (icon_factory, 
+			       "planner-stock-unindent-task",
+			       "24_unindent_task.png");
+	
+	window_add_stock_icon (icon_factory, 
+			       "planner-stock-move-task-up",
+			       "24_task_up.png");
+	
+	window_add_stock_icon (icon_factory, 
+			       "planner-stock-move-task-down",
+			       "24_task_down.png");
+
+	/* Resource icons. */
+	window_add_stock_icon (icon_factory,
+			       "planner-stock-insert-resource",
+			       "/24_insert_resource.png");
+	
+	window_add_stock_icon (icon_factory,
+			       "planner-stock-remove-resource",
+			       "24_remove_resource.png");
+
+	window_add_stock_icon (icon_factory,
+			       "planner-stock-edit-resource",
+			       "24_edit_resource.png");
+	
+	window_add_stock_icon (icon_factory,
+			       "planner-stock-edit-groups",
+			       "24_groups.png");
+}
+
 static void
 window_populate (PlannerWindow *window)
 {
@@ -455,6 +538,7 @@ window_populate (PlannerWindow *window)
 	GtkRadioActionEntry  *r_entries;
 	gchar                *xml_string_tmp, *xml_string;
 	gchar                *str;
+	gchar                *filename;
 	const gchar          *xml_string_full =
 		"<ui>"
 		"<menubar name='MenuBar'>"
@@ -467,7 +551,9 @@ window_populate (PlannerWindow *window)
 		"</ui>";
 
 	priv = window->priv;
-
+	
+	window_add_stock_icons ();
+	
 	priv->ui_box = gtk_vbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (window), priv->ui_box);
 
@@ -494,9 +580,11 @@ window_populate (PlannerWindow *window)
 	gtk_window_add_accel_group (GTK_WINDOW (window),
 				    gtk_ui_manager_get_accel_group (priv->ui_manager));
 
+	filename = mrp_paths_get_ui_dir ("main-window.ui");
 	gtk_ui_manager_add_ui_from_file (priv->ui_manager,
-					 DATADIR "/planner/ui/main-window.ui",
+					 filename ,
 					 NULL);
+	g_free (filename);
 
 	g_object_set (gtk_action_group_get_action (priv->actions, "EditUndo"),
 		      "sensitive", FALSE, 
@@ -508,6 +596,11 @@ window_populate (PlannerWindow *window)
 		      "sensitive", FALSE, 
 		      NULL);
 
+	/* Make the Actions menu always visible even when empty. */
+	g_object_set (gtk_action_group_get_action (priv->actions, "Actions"),
+		      "hide-if-empty", FALSE, 
+		      NULL);
+
 	/* Handle recent file stuff. */
 	priv->recent_view = egg_recent_view_uimanager_new (priv->ui_manager,
 							   "/MenuBar/File/OpenRecent",
@@ -539,16 +632,23 @@ window_populate (PlannerWindow *window)
 
 	gtk_box_pack_end (GTK_BOX (priv->ui_box), hbox, TRUE, TRUE, 0);
 
-	/* Load views. */
+	/* Create views. */
 	priv->view_actions = gtk_action_group_new ("View Actions");
 	gtk_ui_manager_insert_action_group (priv->ui_manager, priv->view_actions, 0);
-	priv->views = planner_view_loader_load (window);
 
+	priv->views = NULL;
+	priv->views = g_list_append (priv->views, planner_gantt_view_new ());
+	priv->views = g_list_append (priv->views, planner_task_view_new ());
+	priv->views = g_list_append (priv->views, planner_resource_view_new ());
+	priv->views = g_list_append (priv->views, planner_usage_view_new ());
+	
 	view_num = 0;
 	xml_string = g_strdup ("");
 	r_entries  = g_new0 (GtkRadioActionEntry, g_list_length (priv->views));
 	for (l = priv->views; l; l = l->next, view_num++ ) {
 		view = l->data;
+
+		planner_view_setup (view, window);
 		
 		view_widget = planner_view_get_widget (view);
 		gtk_widget_show (view_widget);
@@ -706,7 +806,7 @@ get_last_dir (PlannerWindow *window)
 	
 	priv = window->priv;
 	
-	last_dir = planner_conf_get_string (CONF_MAIN_LAST_DIR, NULL);
+	last_dir = planner_conf_get_string (CONF_LAST_DIR, NULL);
 	
 	if (last_dir == NULL) {
 		last_dir = g_strdup (g_get_home_dir ());
@@ -772,7 +872,7 @@ window_open_cb (GtkAction *action,
 			if (filename) {
 				last_dir = g_path_get_dirname (filename);
 				g_free (filename);
-				planner_conf_set_string (CONF_MAIN_LAST_DIR, last_dir, NULL);
+				planner_conf_set_string (CONF_LAST_DIR, last_dir, NULL);
 				g_free (last_dir);
 			}
 		}
@@ -843,11 +943,14 @@ window_print_preview_cb (GtkAction *action,
 	PlannerWindowPriv *priv;
 	GnomePrintConfig  *config;
 	GnomePrintJob     *gpj;
+	GtkWidget         *dialog;
 	GtkWidget         *preview;
 	GList             *views, *l;
 	PlannerView       *view;
 	PlannerPrintJob   *job;
-	gint               n_pages;
+	gboolean           summary;
+	gint               n_pages, n_views;
+	gint               response;
 
 	window = PLANNER_WINDOW (data);
 	priv = window->priv;
@@ -861,6 +964,35 @@ window_print_preview_cb (GtkAction *action,
 
 	views = window_get_print_selection (window);
 	
+	/* Check to be sure there are some views selected */
+	n_views = g_list_length (views);
+	if (n_views == 0) {
+		dialog = planner_print_views_dialog_new (window, priv->views); 
+
+		while (n_views == 0) {
+
+			response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+			if (response == GTK_RESPONSE_CANCEL) {
+				gtk_widget_destroy (dialog);
+				g_object_unref (gpj);
+				return;
+			}
+			else if (response == GTK_RESPONSE_DELETE_EVENT) {
+				gtk_widget_destroy (dialog);
+				g_object_unref (gpj);
+				return;
+			}
+			
+			/* Save printer settings. */
+			planner_print_dialog_save_config (config);
+
+			views = planner_print_dialog_get_print_selection (GTK_DIALOG (dialog), &summary);
+			n_views = g_list_length (views);
+		}
+		gtk_widget_destroy (dialog);
+	}
+
 	n_pages = 0;
 	for (l = views; l; l = l->next) {
 		view = l->data;
@@ -898,13 +1030,13 @@ window_print_cb (GtkAction *action,
 	PlannerWindowPriv *priv;
 	GnomePrintJob     *gpj;
 	GnomePrintConfig  *config;
-	GtkWidget         *dialog;
+	GtkWidget         *dialog, *message;
 	gint               response;
 	gboolean           summary;
 	GList             *views, *l;
 	PlannerView       *view;
 	PlannerPrintJob   *job;
-	gint               n_pages;
+	gint               n_pages, n_views;
 	
 	window = PLANNER_WINDOW (data);
 	priv = window->priv;
@@ -916,22 +1048,43 @@ window_print_cb (GtkAction *action,
 
 	dialog = planner_print_dialog_new (data, gpj, priv->views);
 
-	response = gtk_dialog_run (GTK_DIALOG (dialog));
+	/* n_views is the number of views selected. */
+	n_views = 0;
+	while (n_views == 0) {
 
-	if (response == GTK_RESPONSE_CANCEL) {
-		gtk_widget_destroy (dialog);
-		g_object_unref (gpj);
-		return;
-	}
-	else if (response == GTK_RESPONSE_DELETE_EVENT) {
-		g_object_unref (gpj);
-		return;
-	}
+		response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+		if (response == GTK_RESPONSE_CANCEL) {
+			gtk_widget_destroy (dialog);
+			g_object_unref (gpj);
+			return;
+		}
+		else if (response == GTK_RESPONSE_DELETE_EVENT) {
+			g_object_unref (gpj);
+			return;
+		}
+		
+		/* Save printer settings. */
+		planner_print_dialog_save_config (config);
+
+		views = planner_print_dialog_get_print_selection (GTK_DIALOG (dialog), &summary);
+		n_views = g_list_length (views);
+
+		if (n_views == 0) {
 	
-	/* Save printer settings. */
-	planner_print_dialog_save_config (config);
+			message = gtk_message_dialog_new (GTK_WINDOW (dialog),
+							 GTK_DIALOG_MODAL |
+							 GTK_DIALOG_DESTROY_WITH_PARENT,
+							 GTK_MESSAGE_INFO,
+							 GTK_BUTTONS_OK,
+							 "%s", "Please choose one or more views to print.");
+
+			gtk_dialog_run (GTK_DIALOG (message));
+			gtk_widget_destroy (message);
+
+		}
+	}
 
-	views = planner_print_dialog_get_print_selection (GTK_DIALOG (dialog), &summary);
 	if (summary) {
 		/*g_print ("Print summary\n");*/
 	}
@@ -941,7 +1094,6 @@ window_print_cb (GtkAction *action,
 	n_pages = 0;
 	for (l = views; l; l = l->next) {
 		view = l->data;
-
 		planner_view_print_init (view, job);
 		
 		n_pages += planner_view_print_get_n_pages (view);
@@ -1052,7 +1204,7 @@ window_project_props_cb (GtkAction *action,
 		priv->properties_dialog = planner_project_properties_new (window);
 
 		g_object_add_weak_pointer (G_OBJECT (priv->properties_dialog),
-					   (gpointer *) &priv->properties_dialog);
+					   (gpointer) &priv->properties_dialog);
 
 		gtk_widget_show (priv->properties_dialog);
 	}
@@ -1096,7 +1248,7 @@ window_edit_phases_cb (GtkAction *action,
 		priv->phase_dialog = planner_phase_dialog_new (window);
 		
 		g_object_add_weak_pointer (G_OBJECT (priv->phase_dialog),
-					   (gpointer *) &priv->phase_dialog);
+					   (gpointer) &priv->phase_dialog);
 		
 		gtk_widget_show (priv->phase_dialog);
 	}
@@ -1109,7 +1261,7 @@ window_help_cb (GtkAction *action,
 	GError    *error = NULL;
 	GtkWidget *dialog;
 	
-	if (!gnome_help_display ("planner.xml", NULL, &error)) {
+	if (!planner_util_show_help (&error)) {
 		dialog = gtk_message_dialog_new (GTK_WINDOW (data),
 						 GTK_DIALOG_MODAL |
 						 GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -1124,63 +1276,82 @@ window_help_cb (GtkAction *action,
 	}
 }
 
+static  void
+handle_links (GtkAboutDialog *about, 
+	      const gchar    *link,
+	      gpointer        data)
+{
+	gchar *newlink;
+
+	switch (GPOINTER_TO_INT (data)) {
+	case LINK_TYPE_EMAIL:
+		newlink = g_strdup_printf ("mailto:%s", link);
+		break;
+	case LINK_TYPE_URL:
+		newlink = g_strdup (link);
+		break;
+	default:
+		g_assert_not_reached ();
+	}
+	
+	planner_util_show_url (newlink, NULL);
+	g_free (newlink);
+}
+
 static void
 window_about_cb (GtkAction *action, 
 		 gpointer   data)
 {
-	static GtkWidget *about;
-	GtkWidget        *hbox;
-	GtkWidget        *href;
-
-	const gchar *authors[] = {
+	PlannerWindow *window;
+	gchar         *filename;
+	GdkPixbuf     *pixbuf;
+	const gchar   *authors[] = {
+		"Kurt Maute <kurt@maute.us>",
 		"Richard Hult <richard@imendio.com>",
 		"Mikael Hallendal <micke@imendio.com>",
 		"Alvaro del Castillo <acs@barrapunto.com>",
 		NULL
 	};
-
-	const gchar *documenters[] = {
-		"Kurt Maute <kmaute@yahoo.com>",
+	const gchar   *documenters[] = {
+		"Kurt Maute <kurt@maute.us>",
 		"Alvaro del Castillo <acs@barrapunto.com>",
 		"Pedro Soria-Rodriguez <sorrodp@alum.wpi.edu>",
 		NULL
 	};
-
 	/* I18n: Translators, list your names here, newline separated if there
 	 * are more than one, to appear in the about box.
 	 */
-	const gchar *translator_credits = N_("translator-credits");
-
-	if (about) {
-		gtk_window_present (GTK_WINDOW (about));
-		return;
+	const gchar   *translator_credits = N_("translator-credits");
+	
+	window = PLANNER_WINDOW (data);
+	
+	filename = mrp_paths_get_image_dir ("gnome-planner.png");
+	pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
+	g_free (filename);
+	
+	gtk_about_dialog_set_email_hook ((GtkAboutDialogActivateLinkFunc) handle_links, 
+					 GINT_TO_POINTER (LINK_TYPE_EMAIL), NULL);
+	
+	gtk_about_dialog_set_url_hook ((GtkAboutDialogActivateLinkFunc) handle_links, 
+				       GINT_TO_POINTER (LINK_TYPE_URL), NULL);
+	
+	gtk_show_about_dialog (GTK_WINDOW (window),
+			       "name", "Planner", 
+			       "version", VERSION,
+			       "comments", _("A Project Management application for the GNOME desktop"),
+			       "authors", authors,
+			       "documenters", documenters,
+			       "translator-credits",
+			       strcmp (translator_credits,
+				       _("translator-credits")) != 0 ? _(translator_credits) : NULL,
+			       "website", "http://live.gnome.org/Planner",
+			       "website-label", _("The Planner Homepage"),
+			       "logo", pixbuf,
+			       NULL);
+	
+	if (pixbuf != NULL) {
+		gdk_pixbuf_unref (pixbuf);
 	}
-
-	about = gnome_about_new ("Imendio Planner", VERSION,
-				 "", /*"Copyright \xc2\xa9"*/
-				 _("A Project Management application for the GNOME desktop"),
-				 authors,
-				 documenters,
-				 strcmp (translator_credits, _("translator-credits")) != 0 ? _(translator_credits) : NULL,
-				 NULL);
-
-	gtk_window_set_transient_for (GTK_WINDOW (about),
-				      GTK_WINDOW (data));
-	
-	gtk_window_set_destroy_with_parent (GTK_WINDOW (about), TRUE);
-
-	g_signal_connect (about, "destroy",
-			  G_CALLBACK (gtk_widget_destroyed),
-			  &about);
-	
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (about)->vbox), hbox, FALSE, FALSE, 0);
-	
-	href = gnome_href_new ("http://www.imendio.com/projects/planner/",
-			       _("The Planner Homepage"));
-	gtk_box_pack_start (GTK_BOX (hbox), href, TRUE, FALSE, 0);
-
-	gtk_widget_show_all (about);
 }
 
 static gboolean
@@ -1191,66 +1362,6 @@ window_delete_event_cb (PlannerWindow *window,
 	return TRUE;
 }
 
-/* Stolen from GLib 2.6: g_uri_list_extract_uris (const gchar *uri_list) */
-static gchar **
-uri_list_extract_uris (const gchar *uri_list)
-{
-  GSList *uris, *u;
-  const gchar *p, *q;
-  gchar **result;
-  gint n_uris = 0;
-
-  uris = NULL;
-
-  p = uri_list;
-
-  /* We don't actually try to validate the URI according to RFC
-   * 2396, or even check for allowed characters - we just ignore
-   * comments and trim whitespace off the ends.  We also
-   * allow LF delimination as well as the specified CRLF.
-   *
-   * We do allow comments like specified in RFC 2483.
-   */
-  while (p)
-    {
-      if (*p != '#')
-	{
-	  while (g_ascii_isspace (*p))
-	    p++;
-
-	  q = p;
-	  while (*q && (*q != '\n') && (*q != '\r'))
-	    q++;
-
-	  if (q > p)
-	    {
-	      q--;
-	      while (q > p && g_ascii_isspace (*q))
-		q--;
-
-	      if (q > p)
-		{
-		  uris = g_slist_prepend (uris, g_strndup (p, q - p + 1));
-		  n_uris++;
-		}
-	    }
-	}
-      p = strchr (p, '\n');
-      if (p)
-	p++;
-    }
-
-  result = g_new (gchar *, n_uris + 1);
-
-  result[n_uris--] = NULL;
-  for (u = uris; u; u = u->next)
-    result[n_uris--] = u->data;
-
-  g_slist_free (uris);
-
-  return result;
-}
-
 static void
 window_drag_data_received_cb (GtkWidget        *widget,
 			      GdkDragContext   *context,
@@ -1273,7 +1384,7 @@ window_drag_data_received_cb (GtkWidget        *widget,
 		return;
 	}
 	
-	uris = uri_list_extract_uris (data->data);
+	uris = g_uri_list_extract_uris (data->data);
 
 	i = 0;
 	while (uris[i]) {
@@ -1391,9 +1502,13 @@ window_confirm_exit_run (PlannerWindow *window)
 	}
 	else if (hours == 0) {
 		time_str = 
-			g_strdup_printf (_("If you don't save, changes made "
+			g_strdup_printf (ngettext(
+					   "If you don't save, changes made "
+					   "the last %d minute will be "
+					   "discarded.",
+					   "If you don't save, changes made "
 					   "the last %d minutes will be "
-					   "discarded."), minutes);
+					   "discarded.", minutes), minutes);
 	}
 	else if (hours == 1) {
 		time_str = g_strdup (_("If you don't save, changes made "
@@ -1401,9 +1516,12 @@ window_confirm_exit_run (PlannerWindow *window)
 				       "discarded."));
 	} else {
 		time_str = 
-			g_strdup_printf (_("If you don't save, changes made "
+			g_strdup_printf (ngettext("If you don't save, changes made "
+					   "the last %d hour will be "
+					   "discarded.",
+					   "If you don't save, changes made "
 					   "the last %d hours will be "
-					   "discarded."), hours);
+					   "discarded.", hours), hours);
 	}
 
 	name = window_get_name (window);
@@ -1568,8 +1686,7 @@ window_do_save_as (PlannerWindow *window)
 		success = mrp_project_save_as (window->priv->project, filename, 
 					       FALSE, &error);
 
-		if (!success && error && 
-		    error->code == MRP_ERROR_SAVE_FILE_EXIST) {
+		if (!success && error->code == MRP_ERROR_SAVE_FILE_EXIST) {
 			GtkWidget *dialog;
 			gint       ret;
 			
@@ -1622,7 +1739,7 @@ window_do_save_as (PlannerWindow *window)
 		}
 
 		last_dir = g_path_get_dirname (filename);
-		planner_conf_set_string (CONF_MAIN_LAST_DIR, last_dir, NULL);
+		planner_conf_set_string (CONF_LAST_DIR, last_dir, NULL);
 		g_free (last_dir);
 
 		g_free (filename);
@@ -1669,7 +1786,7 @@ planner_window_new (PlannerApplication *application)
 	PlannerWindow     *window;
 	PlannerWindowPriv *priv;
 	
-	window = g_object_new (PLANNER_TYPE_MAIN_WINDOW, NULL);
+	window = g_object_new (PLANNER_TYPE_WINDOW, NULL);
 	priv = window->priv;
 
 	priv->application = g_object_ref (application);
@@ -1693,7 +1810,6 @@ planner_window_new (PlannerApplication *application)
 			  NULL);
 
 	window_populate (window);
-	
 	window_update_title (window);
 
 	return GTK_WIDGET (window);
@@ -1709,7 +1825,7 @@ planner_window_open (PlannerWindow *window,
 	GtkWidget        *dialog;
 	EggRecentItem    *item;
 	
-	g_return_val_if_fail (PLANNER_IS_MAIN_WINDOW (window), FALSE);
+	g_return_val_if_fail (PLANNER_IS_WINDOW (window), FALSE);
 	g_return_val_if_fail (uri != NULL, FALSE);
 
 	priv = window->priv;
@@ -1785,7 +1901,7 @@ planner_window_open_in_existing_or_new (PlannerWindow *window,
 GtkUIManager *
 planner_window_get_ui_manager (PlannerWindow *window)
 {
-	g_return_val_if_fail (PLANNER_IS_MAIN_WINDOW (window), NULL);
+	g_return_val_if_fail (PLANNER_IS_WINDOW (window), NULL);
 	
 	return window->priv->ui_manager;
 }
@@ -1793,7 +1909,7 @@ planner_window_get_ui_manager (PlannerWindow *window)
 MrpProject *
 planner_window_get_project (PlannerWindow *window)
 {
-	g_return_val_if_fail (PLANNER_IS_MAIN_WINDOW (window), NULL);
+	g_return_val_if_fail (PLANNER_IS_WINDOW (window), NULL);
 	
 	return window->priv->project;
 }
@@ -1801,7 +1917,7 @@ planner_window_get_project (PlannerWindow *window)
 PlannerApplication *
 planner_window_get_application (PlannerWindow  *window)
 {
-	g_return_val_if_fail (PLANNER_IS_MAIN_WINDOW (window), NULL);
+	g_return_val_if_fail (PLANNER_IS_WINDOW (window), NULL);
 	
 	return window->priv->application;
 }
@@ -1813,7 +1929,7 @@ planner_window_check_version (PlannerWindow *window)
 	GtkWidget        *dialog;
 	gint              version;
 
-	g_return_if_fail (PLANNER_IS_MAIN_WINDOW (window));
+	g_return_if_fail (PLANNER_IS_WINDOW (window));
 
 	priv = window->priv;
 	
@@ -1905,7 +2021,7 @@ planner_window_close (PlannerWindow *window)
 	PlannerWindowPriv *priv;
         gboolean           close = TRUE;
         
-	g_return_if_fail (PLANNER_IS_MAIN_WINDOW (window));
+	g_return_if_fail (PLANNER_IS_WINDOW (window));
 
 	priv = window->priv;
 
@@ -1927,7 +2043,7 @@ planner_window_show_day_type_dialog (PlannerWindow *window)
 {
 	PlannerWindowPriv *priv;
 	
-	g_return_if_fail (PLANNER_IS_MAIN_WINDOW (window));
+	g_return_if_fail (PLANNER_IS_WINDOW (window));
 
 	priv = window->priv;
 	
@@ -1937,7 +2053,7 @@ planner_window_show_day_type_dialog (PlannerWindow *window)
 		priv->day_type_dialog = planner_day_type_dialog_new (window);
 
 		g_object_add_weak_pointer (G_OBJECT (priv->day_type_dialog),
-					   (gpointer *) &priv->day_type_dialog);
+					   (gpointer) &priv->day_type_dialog);
 
 		gtk_widget_show (priv->day_type_dialog);
 	}
@@ -1948,7 +2064,7 @@ planner_window_show_calendar_dialog (PlannerWindow *window)
 {
 	PlannerWindowPriv *priv;
 	
-	g_return_if_fail (PLANNER_IS_MAIN_WINDOW (window));
+	g_return_if_fail (PLANNER_IS_WINDOW (window));
 
 	priv = window->priv;
 	
@@ -1958,7 +2074,7 @@ planner_window_show_calendar_dialog (PlannerWindow *window)
 		priv->calendar_dialog = planner_calendar_dialog_new (window);
 		
 		g_object_add_weak_pointer (G_OBJECT (priv->calendar_dialog),
-					   (gpointer *) &priv->calendar_dialog);
+					   (gpointer) &priv->calendar_dialog);
 		
 		gtk_widget_show (priv->calendar_dialog);
 	}
@@ -1986,7 +2102,7 @@ window_recent_tooltip_func (EggRecentItem *item,
 PlannerCmdManager *
 planner_window_get_cmd_manager (PlannerWindow *window)
 {
-	g_return_val_if_fail (PLANNER_IS_MAIN_WINDOW (window), NULL);
+	g_return_val_if_fail (PLANNER_IS_WINDOW (window), NULL);
 	
 	return window->priv->cmd_manager;
 }
@@ -2007,7 +2123,7 @@ window_save_state (PlannerWindow *window)
 		maximized = FALSE;
 	}
 
-	planner_conf_set_bool (CONF_MAIN_WINDOW_MAXIMIZED, maximized, NULL);
+	planner_conf_set_bool (CONF_WINDOW_MAXIMIZED, maximized, NULL);
 
 	/* If maximized don't save the size and position */
 	if (!maximized) {
@@ -2015,12 +2131,12 @@ window_save_state (PlannerWindow *window)
 		int x, y;
 
 		gtk_window_get_size (GTK_WINDOW (window), &width, &height);
-		planner_conf_set_int (CONF_MAIN_WINDOW_WIDTH, width, NULL);
-		planner_conf_set_int (CONF_MAIN_WINDOW_HEIGHT, height, NULL);
+		planner_conf_set_int (CONF_WINDOW_WIDTH, width, NULL);
+		planner_conf_set_int (CONF_WINDOW_HEIGHT, height, NULL);
 
 		gtk_window_get_position (GTK_WINDOW (window), &x, &y);
-		planner_conf_set_int (CONF_MAIN_WINDOW_POS_X, x, NULL);
-		planner_conf_set_int (CONF_MAIN_WINDOW_POS_Y, y, NULL);
+		planner_conf_set_int (CONF_WINDOW_POS_X, x, NULL);
+		planner_conf_set_int (CONF_WINDOW_POS_Y, y, NULL);
 	}
 }
 
@@ -2028,34 +2144,34 @@ static void
 window_restore_state (PlannerWindow *window)
 {
 	PlannerWindowPriv *priv;
-	gboolean exists;
-	gboolean maximized;
-	int      width, height;
-	int      x, y;
+	gboolean           exists;
+	gboolean           maximized;
+	int                width, height;
+	int                x, y;
 
 	priv = window->priv;
 
-	exists = planner_conf_dir_exists (CONF_MAIN_WINDOW_DIR, NULL);
+	exists = planner_conf_dir_exists (CONF_WINDOW_DIR, NULL);
 	
 	if (exists) {	
-		maximized = planner_conf_get_bool (CONF_MAIN_WINDOW_MAXIMIZED,
+		maximized = planner_conf_get_bool (CONF_WINDOW_MAXIMIZED,
 						   NULL);
 	
 		if (maximized) {
 			gtk_window_maximize (GTK_WINDOW (window));
 		} else {
-			width = planner_conf_get_int (CONF_MAIN_WINDOW_WIDTH,
+			width = planner_conf_get_int (CONF_WINDOW_WIDTH,
 						      NULL);
 		
-			height = planner_conf_get_int (CONF_MAIN_WINDOW_HEIGHT,
+			height = planner_conf_get_int (CONF_WINDOW_HEIGHT,
 						       NULL);
 		
 			gtk_window_set_default_size (GTK_WINDOW (window), 
 						     width, height);
 
-			x = planner_conf_get_int (CONF_MAIN_WINDOW_POS_X,
+			x = planner_conf_get_int (CONF_WINDOW_POS_X,
 						  NULL);
-			y = planner_conf_get_int (CONF_MAIN_WINDOW_POS_Y,
+			y = planner_conf_get_int (CONF_WINDOW_POS_Y,
 						  NULL);
 
 			gtk_window_move (GTK_WINDOW (window), x, y);
@@ -2082,9 +2198,8 @@ window_menu_item_select_cb (GtkMenuItem   *proxy,
 			    PlannerWindow *window)
 {
 	PlannerWindowPriv *priv;
-
-	GtkAction *action;
-	gchar     *message;
+	GtkAction         *action;
+	gchar             *message;
 
 	priv = window->priv;
 
